<?php
/*
Plugin Name: WP_Texticon
Version: 0.52
Plugin URI: http://scott.sherrillmix.com/blog/blogger/wp_texticon/
Description: This plugin generates persistent specific icons for each user made out of letters or numbers (e.g. Matrix).
Author: Scott Sherrill-Mix
Author URI: http://scott.sherrillmix.com/blog/
*/

define('WP_TEXTICON_DIR', str_replace('\\','/',preg_replace('@.*([\\\\/]wp-content[\\\\/].*)@','\1',dirname(__FILE__)).'/texticon/'));
define('WP_TEXTICON_DIR_INTERNAL', dirname(__FILE__).'/texticon/');
define('WP_TEXTICON_FONTS_DIR', WP_TEXTICON_DIR_INTERNAL.'fonts/');
define('WP_TEXTICON_MAXWAIT', 5);
define('DEFAULT_TEXTICON_CSS',
'img.texticon{float:left;margin: 1px;}'
);

function texticon_menu() {
	if (function_exists('add_options_page')) {
		add_options_page('Texticon Control Panel', 'Texticon', 1, basename(__FILE__), 'texticon_subpanel');
	}
}

class texticon {
	var $texticon_options;
	var $im;
	var $colors;
	var $size;
	var $transparent=false;
	var $fonts;
	var $gridx;
	var $gridy;
	var $changeArray;
	var $fontArray;
	var $charArray;
	var $chars;
	var $startTime;

	//constructor
	function texticon() {
		$this->texticon_options=texticon_get_options();
		$this->size=320;
		$this->gridx=$this->texticon_options['lettergridx'];
		$this->gridy=$this->texticon_options['lettergridy'];
		$this->chars=$this->texticon_options['chars'];
	}
	




	//use a seed value to determine shape, rotation, and color
	function texticon_set_randomness($seed=""){
		//set seed
		srand(hexdec($seed));
		$basehue=rand()/getrandmax();
		$forecolors=$this->hsl2hex(array($basehue,1,.5));
		$this->colors[1]=imagecolorallocate($this->im, $forecolors[0],$forecolors[1],$forecolors[2]);
		if (array_sum($this->texticon_options['backr'])<0 ||array_sum($this->texticon_options['backg'])<0|| array_sum($this->texticon_options['backb'])<0) {
			$this->colors[0]=imagecolorallocatealpha($this->im,0,0,0,127);
			$this->transparent=true;
			imagealphablending ($this->im,false);
			imagesavealpha($this->im,true);
		} else {
			$backcolors=array(rand($this->texticon_options['backr'][0],$this->texticon_options['backr'][1]), rand($this->texticon_options['backg'][0],$this->texticon_options['backg'][1]), rand($this->texticon_options['backb'][0],$this->texticon_options['backb'][1]));
			$this->colors[0]=imagecolorallocate($this->im, $backcolors[0],$backcolors[1],$backcolors[2]);
		}
		$fonts=$fonts=$this->findfonts();
		for($j=0;$j<$this->gridy;$j++){
			for($i=0;$i<$this->gridx;$i++){
				$this->fontsArray[$i][$j]=$fonts[array_rand($fonts)];
				$this->charArray[$i][$j]=$this->chars[array_rand($this->chars)];
				$this->changeArray[$i][$j]=0;
				$count=0;
				for($k=-1;$k<2;$k++){
					if($this->changeArray[$i+$k][$j-1]!=0&&rand(0,2)==2){
						$this->changeArray[$i][$j]+=$this->changeArray[$i+$k][$j-1];
						$count++;
					}
				}
				if($count>1)$this->changeArray[$i][$j]/=$count;
				if(rand(1,10)==10){
					$this->changeArray[$i][$j]+=(rand(0,200))/500-.2;	
					//echo "Rand: $j $i ".$this->changeArray[$i][$j];
				}
				#echo "Change $i $j: ".$this->changeArray[$i][$j]." <br/>";
				if($this->changeArray[$i][$j]!=0){
					$color=$this->hsl2hex(array($basehue+$this->changeArray[$i][$j],1,.5));
					$this->colors[2][round($this->changeArray[$i][$j]*100)]=imagecolorallocate($this->im, $color[0],$color[1],$color[2]);
				}
			}
		}
		srand();
		return true;
	}

	function findfonts(){
		if(!isset($this->fonts)){
			$dir=WP_TEXTICON_FONTS_DIR;
			$this->fonts=array();
			$noparts=true;
			if (is_dir($dir)&&$dh = opendir($dir)) {
				while (($file = readdir($dh)) !== false) {
					if (is_file($dir.$file)){
						if (strpos($file,'ttf')||strpos($file,'TTF')){
							array_push($this->fonts,$file);
							$noparts=false;
						}
					}
				}
				closedir($dh);
			}
			if ($noparts) return false;
		}
		return $this->fonts;
	}

	function texticon_build($seed='',$altImgText='',$img=true,$outsize='',$write=true,$random=true){
		//make an texticon and return the filepath or if write=false return picture directly
		if (function_exists("gd_info")){
			// init random seed
			if ($random) $id=substr(sha1($seed),0,10);
			else $id=$seed;
			$filename=substr(sha1($id.substr(get_option('admin_email'),0,5)),0,15).'.png';
			if ($outsize=='') $outsize=$this->texticon_options['size'];
			if (!file_exists(WP_TEXTICON_DIR_INTERNAL.$filename)){
				if(!isset($this->startTime))$this->startTime=time();
				#make sure nobody waits more than 5 seconds
				if(time()-$this->startTime>WP_TEXTICON_MAXWAIT) return false;
				$this->im = imagecreatetruecolor($this->size,$this->size);	
				$this->colors = array(imagecolorallocate($this->im, 255,255,255));
				$fonts=$this->findfonts();
				if(!$fonts)return false;
				if ($random) $this->texticon_set_randomness($id);
				else {$this->colors = array(imagecolorallocate($this->im, 255,255,255),imagecolorallocate($this->im, 0,0,0));$this->transparent=false;};
				imagefill($this->im,0,0,$this->colors[0]);
				//Approximate guess at font size
				$fontsize=round($this->size/$this->gridy*1.2);
				if($this->gridx<6){$fontsize*=.85;}
				$lettersize=imagettfbbox($fontsize,0,WP_TEXTICON_FONTS_DIR.$fonts[0],$this->chars[0]);
				$xsize=$lettersize[2]-$lettersize[0];
				$ysize=$lettersize[1]-$lettersize[5];
				for($i=0;$i<$this->gridx;$i++){
					for($j=0;$j<$this->gridy;$j++){
						$text=$this->charArray[$i][$j];
						$font=$this->fontsArray[$i][$j];
						#imagettftext($this->im, rand(60,120), rand(0,359), rand(.15*$this->size,.85*$this->size), rand(.15*$this->size,.85*$this->size), $grey, WP_TEXTICON_DIR."LTYPEB.TTF", $text);
						#imagettftext($this->im, rand(60,100), rand(-10,10), .02*$this->size+.75*$i/($grid-1)*$this->size, .25*$this->size+.73*$j/($grid-1)*$this->size, $grey, WP_TEXTICON_DIR."fonts/".$font, $text);
						if($this->changeArray[$i][$j]!=0)$color=$this->colors[2][round($this->changeArray[$i][$j]*100)];
						else $color=$this->colors[1];
						$border=.01;
						$xpos=($border*$this->size)+($this->size*(1-$border*2)-$xsize)*$i/($this->gridx-1);
						$ypos=($border*$this->size)+$ysize+($this->size*(1-$border*2)-$ysize)*$j/($this->gridy-1);
						$coords=imagettftext($this->im, $fontsize, 0, $xpos, $ypos, $color, WP_TEXTICON_FONTS_DIR.$font, $text);
						//var_dump($coords);
						//var_dump(imagettfbbox ( float size, float angle, string fontfile, string text ));
						//exit;
					}	
				}
				$out = @imagecreatetruecolor($outsize,$outsize);
				imagesavealpha($out,true);
				imagealphablending($out,false);
				imagecopyresampled($out,$this->im,0,0,0,0,$outsize,$outsize,$this->size,$this->size);
				imagedestroy($this->im);
				if ($write){
						$wrote=imagepng($out,WP_TEXTICON_DIR_INTERNAL.$filename);
						if(!$wrote) return false; //something went wrong but don't want to mess up blog layout
				}else{
					header ("Content-type: image/png");
					imagepng($out);
				}
				imagedestroy($out);
			}
			$filename=get_option('siteurl').'/'.WP_TEXTICON_DIR.$filename;
			if($this->texticon_options['gravatar'])
        $filename = "http://www.gravatar.com/avatar.php?gravatar_id=".md5($seed)."&amp;size=$size&amp;default=$filename";
			if ($img){
				$filename='<img class="texticon" src="'.$filename.'" alt="'.str_replace('"',"'",$altImgText).' Texticon Icon" height="'.$outsize.'" width="'.$outsize.'" />';
			}
			return $filename;
		} else { //php GD image manipulation is required
			return false; //php GD image isn't installed but don't want to mess up blog layout
		}
	}

	function HSL2hex($hsl){
		$hue=$hsl[0];
		$saturation=$hsl[1];
		$lightness=$hsl[2];
		if ($saturation == 0){
			$red = $lightness * 255;
			$green = $lightness * 255;
			$blue = $lightness * 255;
		} else {
			if ($lightness < 0.5) $var_2 = $lightness * (1 + $saturation);
			else $var_2 = ($lightness + $saturation) - ($saturation * $lightness);

			$var_1 = 2 * $lightness - $var_2;
			$red = 255 * $this->hue_2_rgb($var_1,$var_2,$hue + (1 / 3));
			$green = 255 * $this->hue_2_rgb($var_1,$var_2,$hue - (1 / 3));
			$blue = 255 * $this->hue_2_rgb($var_1,$var_2,$hue);
		}
		return array($red,$green,$blue);
	}

	function hue_2_rgb($v1,$v2,$vh){
		if ($vh < 0) $vh += 1;
		elseif ($vh > 1) $vh -= 1;
		if ((6 * $vh) < 1) $output=$v1 + ($v2 - $v1) * 6 * $vh;
		elseif ((2 * $vh) < 1) $output=$v2;
		elseif ((3 * $vh) < 2) $output=$v1 + ($v2 - $v1) * ((2 / 3 - $vh) * 6);
		else $output=$v1;
		return($output);
	}

}

//create texticon for later use
$texticon = new texticon;

function texticon_get_options(){
	$texticon_array=get_option('texticon');
	if (!isset($texticon_array['size'])||!isset($texticon_array['backb'])){
		//Set Default Values Here
		$default_array=array('size'=>35,'backr'=>array(0,0),'backg'=>array(0,0),'backb'=>array(0,0),'autoadd'=>1,'gravatar'=>0,'lettergridx'=>8,'lettergridy'=>6,'chars'=>array('0','1'),'css'=>DEFAULT_TEXTICON_CSS);
		add_option('texticon',$default_array,'Options used by Texticon',false);
		$texticon_array=$default_array;
	}
	return($texticon_array);
}

function texticon_subpanel() {
	echo "<div class='wrap'>";
	if (isset($_POST['submit'])) { //update the texticon size option
		$texticon_options=texticon_get_options();
		$texticonsize=intval($_POST['texticonsize']);
		if ($texticonsize > 0 & $texticonsize < 400) $texticon_options['size']=$texticonsize;
		else echo "<div class='error'><p>Please enter an integer for size. Preferably between 30-200.</p></div>";
		$lettergrid=intval($_POST['lettergridx']);
		if ($lettergrid > 0 & $lettergrid < 100) $texticon_options['lettergridx']=$lettergrid;
		else echo "<div class='error'><p>Please enter an integer for number of letters in grid x dimension. Preferably between 3-20.</p></div>";
		$lettergrid=intval($_POST['lettergridy']);
		if ($lettergrid > 0 & $lettergrid < 100) $texticon_options['lettergridy']=$lettergrid;
		else echo "<div class='error'><p>Please enter an integer for number of letters in grid y dimension. Preferably between 3-20.</p></div>";
		$chars=str_split($_POST['chars']);
		if($_POST['chars']!=""&&count($chars)>0)$texticon_options['chars']=$chars;
		else echo "<div class='error'><p>Please enter possible characters.</p></div>"; 
		foreach(array('backr','backg','backb') as $color){//update background color options
			$colorarray=explode('-',$_POST[$color]);
			if($colorarray[0]===""){
				$colorarray[1]=-$colorarray[1];	
				$colorarray[0]=$colorarray[1];
			}
			if (count($colorarray)==1){
				$colorarray[1]=$colorarray[0];
			}
			$colorarray[0]=intval($colorarray[0]);
			$colorarray[1]=intval($colorarray[1]);
			if ($colorarray[0]>=-1 & $colorarray[0]<256 & $colorarray[1]>=-1 & $colorarray[1]<256){
				$texticon_options[$color]=array($colorarray[0],$colorarray[1]);
			}else{
				echo "<div class='error'><p>Please enter a range between two integers for the background color (e.g. 230-255) between -1 and 255. For a single color please enter a single value (e.g. white = 255 for r,g and b).</p></div>";		
			}
		}
		if ($_POST['autoadd'] == 0) $texticon_options['autoadd']=0;
		elseif ($_POST['autoadd'] == 1) $texticon_options['autoadd']=1;
		if ($_POST['gravatar'] == 0) $texticon_options['gravatar']=0;
		elseif ($_POST['gravatar'] == 1) $texticon_options['gravatar']=1;
		update_option('texticon', $texticon_options);
		echo "<div class='updated'><p>Options updated (you may have to clear the texticon cache to see any effect).</p></div>";
	}elseif (isset($_POST['clear'])){ //clear the texticon cache
		$dir=WP_TEXTICON_DIR_INTERNAL;
		if ($dh = opendir($dir)) {
			while (($file = readdir($dh)) !== false) {
				if (is_file($dir.$file) and preg_match('/^.*\.png$/',$file)){
					unlink($dir.$file);
				}
			}
		closedir($dh);
		echo "<div class='updated'><p>Cache cleared.</p></div>";		
		}
	}elseif(isset($_POST['cssreset'])){//reset texticon css to default
		$texticon_options=texticon_get_options();
		$texticon_options['css']=DEFAULT_TEXTICON_CSS;
		update_option('texticon', $texticon_options);
	}elseif(isset($_POST['csssubmit'])){
		$texticon_options=texticon_get_options();
		$texticon_options['css']=$_POST['texticon_css'];
		update_option('texticon', $texticon_options);
	}
	$texticon_options=texticon_get_options();
	//count file
	$texticon_count=0;
	$dir=WP_TEXTICON_DIR_INTERNAL;
	if ($dh = opendir($dir)) {
		while (($file = readdir($dh)) !== false) {
			if (is_file($dir.$file) and preg_match('/^[a-f0-9]*\.png$/',$file)){
				$texticon_count++;
			}
		}
	}
	?>
	<div><p><strong>This is the Texticon options page.</strong></p>
	<p>You currently have <?php echo $texticon_count;?> texticons on your website.</p>
	</div>
	<div class='wrap'>
	<p>Set options here:</p>
	<form method="post" action="options-general.php?page=wp_texticon.php">
		<ul style="list-style-type: none">
	<li><strong>Texticon Size in Pixels</strong> (Default: 35):<br /> 
		<input type="text" name="texticonsize" value="<?php echo $texticon_options['size'];?>"/></li>
	<li><strong>Letters in Grid:</strong> (Default: 8 x 6):<br /> 
		X: <input type="text" name="lettergridx" value="<?php echo $texticon_options['lettergridx'];?>"/>
		Y: <input type="text" name="lettergridy" value="<?php echo $texticon_options['lettergridy'];?>"/></li>
	<li><strong>Possible Characters:</strong> (Default: 01):<br /> 
		<input type="text" name="chars" value="<?php echo implode('',$texticon_options['chars']);?>"/></li>
	<?php $input_back=array('r'=>'style="background:#fee;"','g'=>'style="background:#efe;"','b'=>'style="background:#eef;"');?>
	<li><strong>Background Colors</strong> (enter single value or range. -1,-1,-1 for transparent. Default: 0,0,0):<br />
	R:<input type="text" name="backr" <?php echo $input_back['r'];?> value="<?php echo implode($texticon_options['backr'],'-');?>"/>
	G:<input type="text" name="backg" <?php echo $input_back['g'];?> value="<?php echo implode($texticon_options['backg'],'-');?>"/>
	B:<input type="text" name="backb" <?php echo $input_back['b'];?> value="<?php echo implode($texticon_options['backb'],'-');?>"/></li>
	<li><strong>Automatically Add Texticons to Comments</strong> (adds an Texticon beside commenter names or disable it and edit theme file manually) (default: Auto)<br /> <input type="radio" name="autoadd" value="0" <?php if (!$texticon_options['autoadd']) echo 'checked="checked"';?>/> I'll Do It Myself <input type="radio" name="autoadd" value="1" <?php if ($texticon_options['autoadd']) echo 'checked="checked"';?>/> Add Texticons For Me</li>
	<li><strong>Gravatar Support</strong> (If a commenter has a gravatar use it, otherwise use Texticon) (default: Texticon Only)<br /> <input type="radio" name="gravatar" value="0" <?php if (!$texticon_options['gravatar']) echo 'checked="checked"';?>/> Texticon Only <input type="radio" name="gravatar" value="1" <?php if ($texticon_options['gravatar']) echo 'checked="checked"';?>/> Gravatar + Texticon</li>
	<li><input type="submit" name="submit" value="Set Options"/></li>
	</ul>
	</form>
	<form method="post" action="options-general.php?page=wp_texticon.php">
	<ul style="list-style-type: none"><li>Clear the Texticon Image Cache: <input type="submit" name="clear" value="Clear Cache"/></li></ul>
	</form>
	</div>
	<div class='wrap'><h4>To use Texticon:</h4> <p>Make sure the folder <code>wp-content/plugins/texticon</code> is writable. Texticons should automatically be added beside your commentors names after that. Enjoy.</p> 
	<strong>Testing:</strong><br/>
	<?php if (!is_writable(''.WP_TEXTICON_DIR_INTERNAL)){echo "<div class='error'><p>Texticon needs ".WP_TEXTICON_DIR_INTERNAL." to be writable.</p></div>";}
	if (!function_exists("gd_info")){echo "<div class='error'><p>GD Image library not found. Texticon needs this library.</p></div>";}?>
	<p>A test texticon should be here:<?php echo texticon_build('This is a test','Test');?> and the source URL for this image is <a href="<?php echo texticon_build('This is a test','Test',false);?>">here</a>. If you'd like to change the fonts used for the characters, just upload additional Truetype Fonts (<code>.ttf</code> files) into the <code>texticon/fonts</code> folder.</p>
	<p>If there is no texticon above or there are any other problems, concerns or suggestions please let me know <a href="http://scott.sherrillmix.com/blog/blogger/wp_texticon">here</a>.</p></div>
	<div class="wrap">
	<h4>For advanced users:</h4> <p>Disable the automatic Indenticon placement and put: <br/> <code><?php echo htmlspecialchars('<?php if (isset($texticon)) {echo $texticon->texticon_build($comment->comment_author_email,$comment->comment_author); } ?>');?></code><br/> in the comment loop of your theme comment script (probably <code>comments.php</code>). Or if you're more confident and just want the img URL use:
	<code><?php echo htmlspecialchars('<?php if (isset($texticon)) {echo $texticon->texticon_build($comment->comment_author_email,$comment->comment_author,false); } ?>');?></code></p>
	<p>You can also add any custom css you would like here:</p>
	<form method="post" action="options-general.php?page=wp_texticon.php">
	<ul style="list-style-type: none">
	<li><textarea name="texticon_css" rows="5" cols="70"><?php echo $texticon_options['css'];?></textarea></li>
	<li><input type="submit" name="csssubmit" value="Adjust CSS"/><input type="submit" name="cssreset" value="Return to Default"/></li></ul>
	</form>
	</div>
	</div>
	<div><p>Please see the <a href="http://scott.sherrillmix.com/blog/blogger/wp_texticon/">plugin page</a> if you need more details.</p></div>
	<?php	
}

function texticon_comment_author($output){
		global $comment;
		global $texticon;
		$texticon_options=texticon_get_options();
		if((is_page () || is_single ()) && $texticon_options['autoadd'] && $comment->comment_type!="pingback"&&$comment->comment_type!="trackback"){
			if (isset($texticon)) $output=$texticon->texticon_build($comment->comment_author_email,$comment->comment_author).' '.$output; 
		}
		return $output;
}

function texticon_style() {
	global $texticon;
	$options = texticon_get_options();
	if($css = $options['css']){
?>
<style type="text/css">
<?php echo $css; ?>
</style>
<?php
	}
}

function texticon_build($seed='',$altImgText='',$img=true,$outsize='',$write=true,$random=true){
	global $texticon;
	if (!isset($texticon))$texticon=new texticon();
	if(isset($texticon)){
		return $texticon->texticon_build($seed,$altImgText,$img,$outsize,$write,$random);
	}else return false;
}

if(!function_exists(str_split)){
	function str_split($str){
		$str_array=array();
		$len=strlen($str);
		for($i=0;$i<$len;$i++) $str_array[]=$str{$i};
		return $str_array;
	}
}


//Hooks
add_action('admin_menu', 'texticon_menu');
add_filter('get_comment_author','texticon_comment_author');
add_action('wp_head', 'texticon_style');


?>